
% Use this to generate data automagically.

% This is easier than it looks.

% The diagnostic stuff will come out on stderr, and the document on
% stdout.
% So just pipe stdout of this to a file and you have test data!

% For ensuring larger files, try this handy script (set the size limit
% as you wish).
%
% #!/bin/sh
% size=0
% i=0
% 
% while [ $i -lt 10 ] ; do
%	while [ $size -lt 200 ] ; do
%		./generate_data > zzz
%		size=`ls -l zzz | row 5`
%	done
%	mv zzz zz_generated_data_$i
%	echo "found a result: zz_generated_data_$i"
%	sleep 2
%	size=0
% 	i=`expr $i + 1`
% done

:- module generate_data.


:- interface.

:- use_module io.

:- pred main(io__state, io__state).
:- mode main(di, uo) is det.


:- implementation.

:- import_module int, list, std_util.
:- import_module qcheck, rnd, smlng_data, output.

main -->
        qcheck(qcheck__f(process_document), "user function", 1, [], [], 
	       [{type_of(4), gen_int},
	        {type_of("abc"), gen_string}]).

:- func gen_int(type_desc, list(frequency),
		list({type_desc, list(frequency)}),
                list(user_gen_type), rnd, rnd) = univ.
:- mode gen_int(in, in, in, list_skel_in(user_gen_inst), in, out) = out is det.
gen_int(_, _, _, _, RS0, RS) = Univ :-
	rnd__irange(0, 9, X, RS0, RS),
	Univ = univ(X).

:- func gen_string(type_desc, list(frequency),
		list({type_desc, list(frequency)}),
                list(user_gen_type), rnd, rnd) = univ.
:- mode gen_string(in, in, in, list_skel_in(user_gen_inst), in, out) = out is det.
gen_string(_, _, _, _, RS, RS) = Univ :-
	Univ = univ("abc").

:- func process_document(list(document)) = property.
process_document(X) = [yes] :-
	unsafe_perform_io(output_to_file(X)).



:- pred output_to_file(list(document)::in, io__state::di, io__state::uo) is det.
output_to_file(Docs) -->
	io__write_list(Docs, "", output),
	io__stderr_stream(StdErr),
	io__set_output_stream(StdErr, _).

:- pred unsafe_perform_io(pred(io__state, io__state)).
:- mode unsafe_perform_io(pred(di, uo) is det) is det.

:- pragma promise_pure(unsafe_perform_io/1). % XXX this is a lie

unsafe_perform_io(Goal) :-
        impure make_io_state(IOState0),
	Goal(IOState0, IOState),
	impure consume_io_state(IOState).

:- impure pred make_io_state(io__state::uo) is det.
:- pragma foreign_proc("C", make_io_state(_IO::uo),
                [will_not_call_mercury, thread_safe], "").
:- pragma foreign_proc("MC++", make_io_state(_IO::uo),
                [will_not_call_mercury, thread_safe], "").

:- impure pred consume_io_state(io__state::di) is det.
:- pragma foreign_proc("C", consume_io_state(_IO::di),
                [will_not_call_mercury, thread_safe], "").
:- pragma foreign_proc("MC++", consume_io_state(_IO::di),
                [will_not_call_mercury, thread_safe], "").



