:- module output.

:- interface.

:- import_module smlng_data.
:- import_module io.

:- pred output(document::in, io__state::di, io__state::uo) is det.

:- implementation.

:- import_module attrs.
:- import_module char, io, list, require, string.

output(doc(Doc1, Doc2)) -->
	output(Doc1),
	output(Doc2).

output(text(Text)) -->
	io__write_list(Text, "", output_text).
	
output(style(Style, Doc)) -->
	io__write_string("<"), 
	output_style(Style), 
	io__write_string(">"), 
	output(Doc),
	io__write_string("</"), 
	output_style(Style), 
	io__write_string(">"). 

output(size(Size, Doc)) -->
	io__write_string("<"), 
	io__write_int(Size), 
	io__write_string(">"), 
	output(Doc),
	io__write_string("</"), 
	io__write_int(Size), 
	io__write_string(">"). 

output(colour(Colour, Doc)) -->
	io__write_string("<"), 
	output_colour(Colour), 
	io__write_string(">"), 
	output(Doc),
	io__write_string("</"), 
	output_colour(Colour), 
	io__write_string(">"). 

:- pred output_text(textchar::in, io__state::di, io__state::uo) is det.
output_text(s(String)) -->
	io__write_string(String).
output_text(cr) -->
	io__write_string("\n").
output_text(lf) -->
	io__write_string("\r").
output_text(tab) -->
	io__write_string("\t").
output_text(spc) -->
	io__write_string(" ").

:- pred output_style(style::in, io__state::di, io__state::uo) is det.
output_style(b) --> io__write_string("B").
output_style(em) --> io__write_string("EM").
output_style(i) --> io__write_string("I").
output_style(pl) --> io__write_string("PL").
output_style(s) --> io__write_string("S").
output_style(tt) --> io__write_string("TT").
output_style(u) --> io__write_string("U").


:- pred output_colour(colour::in, io__state::di, io__state::uo) is det.
output_colour(r) --> io__write_string("r").
output_colour(g) --> io__write_string("g").
output_colour(b) --> io__write_string("b").
output_colour(c) --> io__write_string("c").
output_colour(m) --> io__write_string("m").
output_colour(y) --> io__write_string("y").
output_colour(k) --> io__write_string("k").
output_colour(w) --> io__write_string("w").
output_colour(root_colour) --> [].
