%------------------------------------------------------------------------------%
% spans.m
% Ralph Becket <rbeck@microsoft.com>
% Thu Jul 26 19:07:20 BST 2001
% vim: ts=4 sw=4 et tw=0 wm=0 ff=unix ft=mercury
%
% We use a list of {start, end} pairs.
%
% XXX A pair of lists [S0, S1, S2, ...]
%                     [E0, E1, E2, ...]
% would be more economical.  Would it be worth it?
%
% XXX Do we need some structure to convert these things
% into balanced trees for fast lookup operations?
%
%------------------------------------------------------------------------------%

:- module spans.

:- interface.

:- import_module list, int.



:- type span == {int, int}.             % {start index, end index}

:- type spans == list(span).            % Ordered by increasing start index.



    % Converts adjacent spans into one.
    %
:- func merge_spans(spans) = spans.



    % Decides whether the RH span encloses the LH span.
    %
:- pred (span::in) `inside` (span::in) is semidet.



%------------------------------------------------------------------------------%
%------------------------------------------------------------------------------%

:- implementation.

%------------------------------------------------------------------------------%

merge_spans([])                     = [].

merge_spans([Span])                 = ([Span]).

merge_spans([Span1, Span2 | Spans]) =
    reverse(mspns(Span1, [Span2 | Spans], [])).



:- func mspns(span, spans, spans) = spans.

mspns(Span, [], Spans)                      = [Span | Spans].

mspns({S0, E0}, [{S1, E1} | Spans], Spans0) =
    ( if E0 + 1 = S1 then mspns({S0, E1}, Spans, Spans0)
                     else mspns({S1, E1}, Spans, [{S0, E0} | Spans0])
    ).

%------------------------------------------------------------------------------%

{S0, E0} `inside` {S1, E1} :-
    S1 =< S0, E0 =< E1.

%------------------------------------------------------------------------------%
%------------------------------------------------------------------------------%
