:- module validator.

:- interface.

:- import_module smlng_data.
:- import_module std_util.

:- type error == maybe(string).

:- pred validate(document::in, document::in, error::out) is det.

:- implementation.

:- import_module attrs.
:- import_module bool, list.

:- type prevchar == pair(textchar, attrs).
:- type prevchars == list(prevchar).
	

validate(DocA, DocB, Error) :-
	output_chars(DocA, root_attrs, [s("") - root_attrs], OutputCharsA),
	output_chars(DocB, root_attrs, [s("") - root_attrs], OutputCharsB),
	( OutputCharsA = OutputCharsB ->
		Error = no
	;
		Error = yes("XXX")
	).

:- pred output_chars(document::in, attrs::in, prevchars::in,
		prevchars::out) is det.

output_chars(Doc, Attrs, PrevChars0, PrevChars) :-
	Doc = doc(DocA, DocB),
	output_chars(DocA, Attrs, PrevChars0, PrevChars1),
	output_chars(DocB, Attrs, PrevChars1, PrevChars).

output_chars(Doc, Attrs, PrevChars0, PrevChars) :-
	Doc = style(_, SubDoc),
	output_chars(SubDoc, apply_tag(Doc, Attrs), PrevChars0, PrevChars).
output_chars(Doc, Attrs, PrevChars0, PrevChars) :-
	Doc = size(_, SubDoc),
	output_chars(SubDoc, apply_tag(Doc, Attrs), PrevChars0, PrevChars).
output_chars(Doc, Attrs, PrevChars0, PrevChars) :-
	Doc = colour(_, SubDoc),
	output_chars(SubDoc, apply_tag(Doc, Attrs), PrevChars0, PrevChars).
output_chars(Doc, Attrs, PrevChars0, PrevChars) :-
	Doc = text(TextChars),
	( TextChars = [],
		PrevChars = PrevChars0
	; TextChars = [TextChar | Rest],
		PrevChar = list__det_head(PrevChars0),
		output_textchar(TextChar, Attrs, PrevChar, MaybeNextPrevChar),
		( MaybeNextPrevChar = yes(NewPrevChar),
			PrevChars1 = [NewPrevChar | PrevChars0]
		; MaybeNextPrevChar = no,
			PrevChars1 = PrevChars0
		),
		output_chars(text(Rest), Attrs, PrevChars1, PrevChars)
	).

:- pred output_textchar(textchar::in, attrs::in,
		prevchar::in, maybe(prevchar)::out) is det.

output_textchar(s(Str), Attr, _, yes(s(Str) - Attr)).
output_textchar(cr, Attr, PrevChar, MaybeNextPrevChar) :-
	output_textchar_ws(tab, Attr, PrevChar, MaybeNextPrevChar).
output_textchar(lf, Attr, PrevChar, MaybeNextPrevChar) :-
	output_textchar_ws(tab, Attr, PrevChar, MaybeNextPrevChar).
output_textchar(tab, Attr, PrevChar, MaybeNextPrevChar) :-
	output_textchar_ws(tab, Attr, PrevChar, MaybeNextPrevChar).
output_textchar(spc, Attr, PrevChar, MaybeNextPrevChar) :-
	output_textchar_ws(spc, Attr, PrevChar, MaybeNextPrevChar).


:- pred output_textchar_ws(textchar::in, attrs::in,
		prevchar::in, maybe(prevchar)::out) is det.

output_textchar_ws(TextChar, Attr, PrevChar, NextPrevChar) :-
	SpaceAttr = adjust_for_space(Attr),
	( Attr ^ tt = yes ->
		NextPrevChar = yes(TextChar - SpaceAttr)
	; 
		( PrevChar = spc - PrevSpaceAttr, PrevSpaceAttr = SpaceAttr ->
			% Do nothing
			NextPrevChar = no
		;
			% Output a spc
			NextPrevChar = yes(spc - SpaceAttr)
		)
	).
